'use strict'

var _interopRequireDefault = require('@babel/runtime/helpers/interopRequireDefault')

Object.defineProperty(exports, '__esModule', {
  value: true,
})
exports.expression = expression
exports.statement = statement
exports.statements = statements

var _toConsumableArray2 = _interopRequireDefault(
  require('@babel/runtime/helpers/toConsumableArray')
)

var _compileReplacement = _interopRequireDefault(
  require('../compileReplacement')
)

var _convertToExpression = _interopRequireDefault(
  require('../convertReplacement/convertToExpression')
)

var _convertStatementReplacement = _interopRequireDefault(
  require('../convertReplacement/convertStatementReplacement')
)

var _ensureArray = _interopRequireDefault(require('../util/ensureArray'))

var _Astx = _interopRequireDefault(require('../Astx'))
/**
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */
function statements(code) {
  var _this = this

  var captures = {}
  var arrayCaptures = {}
  var varNames = []

  for (
    var _len = arguments.length,
      nodes = new Array(_len > 1 ? _len - 1 : 0),
      _key = 1;
    _key < _len;
    _key++
  ) {
    nodes[_key - 1] = arguments[_key]
  }

  for (var i = 0; i < nodes.length; i++) {
    if (typeof nodes[i] === 'string') {
      varNames.push(nodes[i])
    } else if (Array.isArray(nodes[i])) {
      var name = '$$tpl___'.concat(i)
      arrayCaptures[name] = nodes[i].map(function (n) {
        return n instanceof _Astx['default'] ? n.node : n
      })
      varNames.push(name)
    } else {
      if (nodes[i] instanceof _Astx['default']) {
        var astx = nodes[i]

        if (astx.size > 1) {
          var _name = '$$tpl___'.concat(i)

          arrayCaptures[_name] = (0, _toConsumableArray2['default'])(astx.nodes)
          varNames.push(_name)
        } else {
          var _name2 = '$tpl___'.concat(i)

          captures[_name2] = astx.node
          varNames.push(_name2)
        }
      } else {
        var _name3 = '$tpl___'.concat(i)

        captures[_name3] = nodes[i]
        varNames.push(_name3)
      }
    }
  }

  var src = (0, _toConsumableArray2['default'])(
    (0, _ensureArray['default'])(code)
  ).reduce(function (result, elem, i) {
    return result + varNames[i - 1] + elem
  })
  if (
    !nodes.length ||
    nodes.every(function (n) {
      return typeof n === 'string'
    })
  )
    return this.parseStatements(src)
  var result = (0, _compileReplacement['default'])(
    this.parseStatements(src).map(function (n) {
      return new _this.t.NodePath(n)
    }),
    {
      backend: this,
    }
  ).generate({
    captures: captures,
    arrayCaptures: arrayCaptures,
  })
  return (0, _ensureArray['default'])(result).map(
    _convertStatementReplacement['default']
  )
}

function statement(template) {
  var _this$template

  for (
    var _len2 = arguments.length,
      nodes = new Array(_len2 > 1 ? _len2 - 1 : 0),
      _key2 = 1;
    _key2 < _len2;
    _key2++
  ) {
    nodes[_key2 - 1] = arguments[_key2]
  }

  var result = (_this$template = this.template).statements.apply(
    _this$template,
    [template].concat(nodes)
  )

  if (result.length !== 1) {
    throw new Error('code is not a statement')
  }

  return result[0]
}

function expression(code) {
  var captures = {}
  var arrayCaptures = {}
  var varNames = []

  for (
    var _len3 = arguments.length,
      nodes = new Array(_len3 > 1 ? _len3 - 1 : 0),
      _key3 = 1;
    _key3 < _len3;
    _key3++
  ) {
    nodes[_key3 - 1] = arguments[_key3]
  }

  for (var i = 0; i < nodes.length; i++) {
    if (typeof nodes[i] === 'string') {
      varNames.push(nodes[i])
    } else if (Array.isArray(nodes[i])) {
      var name = '$$tpl___'.concat(i)
      arrayCaptures[name] = nodes[i]
      varNames.push(name)
    } else {
      var _name4 = '$tpl___'.concat(i)

      captures[_name4] = nodes[i]
      varNames.push(_name4)
    }
  }

  var src = (0, _toConsumableArray2['default'])(
    (0, _ensureArray['default'])(code)
  ).reduce(function (result, elem, i) {
    return result + varNames[i - 1] + elem
  })
  if (
    !nodes.length ||
    nodes.every(function (n) {
      return typeof n === 'string'
    })
  )
    return this.parseExpression(src)
  var result = (0, _compileReplacement['default'])(
    new this.t.NodePath(this.parseExpression(src)),
    {
      backend: this,
    }
  ).generate({
    captures: captures,
    arrayCaptures: arrayCaptures,
  })
  var expression

  if (Array.isArray(result)) {
    if (result.length !== 1) {
      throw new Error('code is not an expression: '.concat(src))
    }

    expression = (0, _convertToExpression['default'])(result[0])
  } else {
    expression = (0, _convertToExpression['default'])(result)
  }

  if (!expression) {
    throw new Error('code is not an expression: '.concat(src))
  }

  return expression
}